require 'test_helper'

class Gitmain::RepositoryTest < Minitest::Test
  def setup
    super
    Gitmain::Repository.instance_variable_set(:@default_branch, nil)
    Gitmain::Repository.instance_variable_set(:@dirname, nil)
  end

  def test_repository_default_branch_returns_default_value
    assert_equal 'master', Gitmain::Repository.default_branch
  end

  def test_repository_default_branch_receives_and_returns_value
    assert_equal 'production', Gitmain::Repository.default_branch('production')
    assert_equal 'production', Gitmain::Repository.default_branch
  end

  def test_repository_dirname_must_receive_a_value
    assert_raises(ArgumentError) do
      Gitmain::Repository.dirname
    end
  end

  def test_repository_dirname_receives_and_returns_value
    assert_equal '/test', Gitmain::Repository.dirname('/test')
    assert_equal '/test', Gitmain::Repository.dirname
  end

  def test_repository_init_requires_name
    assert_raises(ArgumentError) do
      Gitmain::Repository.init('')
    end
  end

  def test_repository_init_returns_new_instance
    Gitmain::Repository.dirname(repo_dirname)
    repository = Gitmain::Repository.init('asdf-123')
    assert repository.is_a?(Gitmain::Repository)
    assert_equal repository.name, 'asdf-123'
  end

  def test_initialize_requires_name
    assert_raises(ArgumentError) do
      Gitmain::Repository.new('', '')
    end
  end

  def test_initialize_requires_repository_name
    assert_raises(ArgumentError) do
      Gitmain::Repository.dirname(repo_dirname)
      Gitmain::Repository.new('asdf-123', '')
    end
  end

  def test_initialize_requires_bare_repository_name
    assert_raises(ArgumentError) do
      full_repository
    end
  end

  def test_initialize_requires_branch
    assert_raises(ArgumentError) do
      Gitmain::Repository.dirname(repo_dirname)
      Gitmain::Repository.new('repository', '')
    end
  end

  def test_initialize_empty_repository_sets_rugged_repo
    empty_repository do |repository|
      rugged_repo = repository.instance_variable_get(:@_repo)
      assert_equal rugged_repo.class, Rugged::Repository
      assert_equal rugged_repo.path, File.join(repo_dirname, 'repository.git/')
    end
  end

  def test_initialize_setup_repository_sets_rugged_repo_and_branch
    setup_repository do |repository|
      rugged_repo = repository.instance_variable_get(:@_repo)
      assert_equal rugged_repo.class, Rugged::Repository
      assert_equal rugged_repo.path, File.join(repo_dirname, 'repository.git/')
      rugged_branch = repository.instance_variable_get(:@_branch)
      assert_equal rugged_branch.class, Rugged::Branch
      assert_equal rugged_branch.name, 'master'
    end
  end

  def test_initialize_setup_repository_requires_known_branch
    assert_raises(ArgumentError) do
      setup_repository('dev')
    end
  end

  def test_initialized_values_are_retrievable
    empty_repository do |repository|
      assert_equal 'repository', repository.name
      assert_equal 'master', repository.branch
    end
  end

  def test_initialize_allows_default_branch
    empty_repository do |repository|
      assert_equal Gitmain::Repository.default_branch, repository.branch
    end
  end

  def test_commit_returns_nil_if_empty
    empty_repository do |repository|
      assert_nil repository.commit
    end
  end

  def test_commit_returns_commit_instance
    setup_repository do |repository|
      rugged_repo = repository.instance_variable_get(:@_repo)
      assert repository.commit.is_a?(Gitmain::Commit)
      assert_equal repository.commit.repository, repository
      assert_equal repository.commit.id, rugged_repo.branches['master'].target.oid
    end
  end

  def test_commit_id_returns_nil_if_empty
    empty_repository do |repository|
      assert_nil repository.commit_id
    end
  end

  def test_commit_id_returns_commit_sha
    setup_repository do |repository|
      rugged_repo = repository.instance_variable_get(:@_repo)
      assert_equal rugged_repo.branches['master'].target.oid, repository.commit_id
    end
  end

  def test_checkout_sets_branch
    setup_repository do |repository|
      assert_equal 'dev', repository.checkout('dev')
      assert_equal 'dev', repository.branch
      rugged_repo = repository.instance_variable_get(:@_repo)
      assert_equal rugged_repo.branches['master'].target.tree_id, rugged_repo.branches['dev'].target.tree_id
    end
  end

  def test_rename_sets_name
    empty_repository do |repository|
      assert_equal 'renamed', repository.rename('renamed')
      assert_equal 'renamed', repository.name
    end
  end

  def test_rename_affects_repository
    empty_repository do |repository|
      repository.rename('renamed')
      rugged_repo = repository.instance_variable_get(:@_repo)
      assert_equal rugged_repo.path, File.join(repo_dirname, 'renamed.git/')
    end
  end

  def test_setup?
    empty_repository do |repository|
      refute repository.setup?
    end

    setup_repository do |repository|
      assert repository.setup?
    end
  end

  def test_tree_returns_nil_if_empty
    empty_repository do |repository|
      assert_nil repository.tree
    end
  end

  def test_tree_returns_tree_instance
    setup_repository do |repository|
      rugged_repo = repository.instance_variable_get(:@_repo)
      assert repository.tree.is_a?(Gitmain::Tree)
      assert_equal repository.tree.repository, repository
      assert_equal repository.tree.id, rugged_repo.branches['master'].target.tree_id
    end
  end

  def test_tree_id_returns_nil_if_empty
    empty_repository do |repository|
      assert_nil repository.tree_id
    end
  end

  def test_tree_id_returns_tree_sha
    setup_repository do |repository|
      rugged_repo = repository.instance_variable_get(:@_repo)
      assert_equal repository.tree_id, rugged_repo.branches['master'].target.tree_id
    end
  end
end
